@@ -19,6 +19,7 @@
 
 #include <stddef.h>
 #include <dirent.h>
+#include <stdbool.h>
 
 struct btrfs_fs_devices;
 
@@ -27,15 +28,10 @@ int check_mounted_where(int fd, const char *file, char *where, int size,
 			bool noscan);
 int check_mounted(const char* file);
 int get_btrfs_mount(const char *dev, char *mp, size_t mp_size);
-int open_path_or_dev_mnt(const char *path, DIR **dirstream, int verbose);
 
-int open_file_or_dir3(const char *fname, DIR **dirstream, int open_flags);
-int open_file_or_dir(const char *fname, DIR **dirstream);
-
-int btrfs_open(const char *path, DIR **dirstream, int verbose, int dir_only);
-int btrfs_open_dir(const char *path, DIR **dirstream, int verbose);
-int btrfs_open_file_or_dir(const char *path, DIR **dirstream, int verbose);
-
-void close_file_or_dir(int fd, DIR *dirstream);
+int btrfs_open_path(const char *path, bool read_write, bool dir_only);
+int btrfs_open_file_or_dir(const char *path);
+int btrfs_open_dir(const char *path);
+int btrfs_open_mnt(const char *path);
 
 #endif
