@@ -385,10 +385,12 @@ void read_super_blocks(char* device, file_system_info* fs_info)
     fs_info->block_size  = fat_sb.sector_size;
     fs_info->totalblock  = total_sector;
     fs_info->usedblocks  = bused;
+    fs_info->superBlockUsedBlocks = fs_info->usedblocks;
     fs_info->device_size = total_sector * fs_info->block_size;
     log_mesg(2, 0, 0, fs_opt.debug, "%s: Block Size:%i\n", __FILE__, fs_info->block_size);
     log_mesg(2, 0, 0, fs_opt.debug, "%s: Total Blocks:%llu\n", __FILE__, fs_info->totalblock);
     log_mesg(2, 0, 0, fs_opt.debug, "%s: Used Blocks:%llu\n", __FILE__, fs_info->usedblocks);
+    log_mesg(2, 0, 0, fs_opt.debug, "%s: superBlockUsedBlocks:%llu\n", __FILE__, fs_info->superBlockUsedBlocks);
     log_mesg(2, 0, 0, fs_opt.debug, "%s: Device Size:%llu\n", __FILE__, fs_info->device_size);
 
     fs_close();
